/*
 *	Title:		o3find
 *	Version:	0.8
 *	Created by:	Paolo Copello
 *	Home page:	http://web.tiscali.it/fanelia
 *	
 *	Copyright (C) 2003-2005 Paolo Copello
 *	
 *	******************************************
 *	This file is part of o3find
 *	******************************************
 *	
 *	o3find is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	any later version.
 *	
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *	
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *	
 */





// suppress "truncated debug info" warnings (VC++ specific)
#ifdef _MSC_VER
#pragma warning (disable: 4786)
#pragma warning (disable: 4788)
#endif // _MSC_VER


#include <string>
#include <list>
#include <vector>
#include <algorithm>

using namespace std;

#include <stdio.h>
#include <io.h>


#include "config.h"
#include "fnutils.h"

#include "fnfilefind.h"


namespace Fanel
{


static const char FANEL_FORWARD_SLASH = '/';
static const char FANEL_BACK_SLASH = '\\';

// Define FANEL_USE_WINDOWS_BACKSLASH to use "\"
// (preferred in DOS/Windows systems)

#ifdef FANEL_USE_WINDOWS_BACKSLASH
#define FANEL_PREFERRED_SLASH		FANEL_BACK_SLASH
#else
#define FANEL_PREFERRED_SLASH		FANEL_FORWARD_SLASH
#endif


/** FileFinder constructor */
FileFinder::FileFinder()
: _bRunning(false) { }

/** If needed, append a slash at the end of sDirectory.
	The function uses a (DOS/Win) backslash if FANEL_USE_WINDOWS_BACKSLASH is defined, 
	otherwise it uses a (Unix/Linux) forward slash. */
void FileFinder::AppendSlashToDir(string& sDirectory)
{
	if (sDirectory.empty())
		return;

	if (sDirectory.at(sDirectory.length()-1) != FANEL_FORWARD_SLASH
		&& sDirectory.at(sDirectory.length()-1) != FANEL_BACK_SLASH)
		sDirectory += FANEL_PREFERRED_SLASH;
}

/** Find sub-directories in a directory. sDirectory MUST have the end slash. */
void FileFinder::FindDirectoriesInDir(const string& sDirectory, list<string>& lst)
{
	struct _finddata_t c_file;
	long hFile;

	string sPath = sDirectory + "*.*";

	if( (hFile = _findfirst( sPath.c_str(), &c_file )) != -1L )
	{
		do
		{
			if ( (c_file.attrib & _A_SUBDIR) && strcmp(".", c_file.name)!=0 
					&& strcmp("..", c_file.name)!=0  )
				lst.push_back( sDirectory + c_file.name);
		} while ( _findnext( hFile, &c_file ) == 0 );
		_findclose( hFile );
	}
}


/** Find files in a single directory. Use only one filter. */
void FileFinder::FindFilesSingleDirOneFilter(const string& sDirectory, const string& sFilter, list<string>& lst) /*, bool bCheckUnique)*/
{
	struct _finddata_t c_file;
	long hFile;

	string sPath = sDirectory + sFilter;

	if( (hFile = _findfirst( sPath.c_str(), &c_file )) != -1L )
	{
		do
		{
			if (  !(c_file.attrib & _A_SUBDIR ) )
			{
				lst.push_back( sDirectory + c_file.name);
			}
		} while ( _findnext( hFile, &c_file ) == 0 );
		_findclose( hFile );
	}

}

/** Find files in a single directory. Use many filters. */
void FileFinder::FindFilesSingleDir(const string& sDirectory, const list<string>& sFilterList, list<string>& lst)
{
	if (sFilterList.empty())
		return;

	list<string>::const_iterator it = sFilterList.begin();
	
	for( ; it != sFilterList.end(); ++it)
	{
		FindFilesSingleDirOneFilter(sDirectory, *it, lst);
	}
}

/** Recursive function to find files. Set bSubDir = true to search subdirectories.
	Returns false if the search was stopped (for multithreaded applications)  */
bool FileFinder::RecursiveFind(const string& sDirectory, const list<string>& sFilterList, list<string>& lst, bool bSubDir)
{
	string sUsedDir = Fanel::Str::Trim(sDirectory);
	AppendSlashToDir(sUsedDir);

	FindFilesSingleDir(sUsedDir, sFilterList,  lst);

	if (!_bRunning)
		return false;

	if (bSubDir)
	{
		list<string> directories;
		FindDirectoriesInDir(sUsedDir, directories);

		for(list<string>::const_iterator it = directories.begin(); it!=directories.end(); ++it)
		{
			// call itself! 
			if (! Find ( (*it)+FANEL_PREFERRED_SLASH , sFilterList, lst, true) )
				return false;
		}
	}
	return true;
}


/** Find files using one filter and one directory. Set bSubDir = true to search subdirectories
	Returns false if the search was stopped (for multithreaded applications) */
bool FileFinder::Find(const string& sDirectory, const string& sFilter, list<string>& lst, bool bSubDir)
{
	list<string> tmpList;
	tmpList.push_back(sFilter);

	_bRunning = true;

	bool bNormal = RecursiveFind(sDirectory, tmpList, lst, bSubDir);

	_bRunning = false;

	return bNormal;
}

/** Find files using many filters and many directories. Set bSubDir = true to search subdirectories.
	Returns false if the search was stopped (for multithreaded applications) */
bool FileFinder::Find(const list<string>& directoryList, const list<string>& sFilterList, list<string>& lst, bool bSubDir)
{
	_bRunning = true;

	bool bNormal = true;
	list<string>::const_iterator it = directoryList.begin();
	for(; bNormal && it!=directoryList.end(); ++it)
	{
		bNormal = RecursiveFind(*it, sFilterList, lst, bSubDir);
	}

	_bRunning = false;

	return bNormal;
}

/** Find files using many filters and one directory. Set bSubDir = true to search subdirectories.
	Returns false if the search was stopped (for multithreaded applications) */
bool FileFinder::Find(const string& sDirectory, const list<string>& sFilterList, list<string>& lst, bool bSubDir)
{
	_bRunning = true;

	bool bNormal = RecursiveFind(sDirectory, sFilterList, lst, bSubDir);

	_bRunning = false;

	return bNormal;
}


/** Stop search. Can be used by multithreaded applications. */
void FileFinder::Stop()
{
	_bRunning = false;
}



} // end namespace Fanel

